#include "gameloop.hpp"

GAME_STATE ProcessEvents(GAME_STATE currentState, SDL_Event &event, Player &pacman)
{
	GAME_STATE state = currentState;
	switch (event.type) 
	{
		case SDL_KEYDOWN:
			switch(event.key.keysym.sym) 
			{
				case SDLK_LEFT:
					pacman.moveLeft();
					break;
				case SDLK_RIGHT:
					pacman.moveRight();
					break;
				case SDLK_UP:
					pacman.moveUp();
      				break;
      			case SDLK_DOWN:
      				pacman.moveDown();
      				break;
      			case SDLK_q:
      				state = GAME_STATE_TITLE;
      				break;
      			case SDLK_e:
      				state = GAME_STATE_QUIT;
      				break;
    			case SDLK_w:
      				state = GAME_STATE_WIN;
      				break;
    			case SDLK_l:
      				state = GAME_STATE_LOSE;
      				break;
      			default:
      				break;
    		}
			break;
		case SDL_KEYUP:
			switch( event.key.keysym.sym )
			{
    			case SDLK_LEFT:
      				pacman.stopLeft();
      				break;
    			case SDLK_RIGHT:
      				pacman.stopRight();
      				break;
    			case SDLK_UP:
      				pacman.stopUp();
      				break;
    			case SDLK_DOWN:
      				pacman.stopDown();
      				break;
    			default:
      				break;
    		}
			break;
		case SDL_QUIT:
    		state = GAME_STATE_QUIT;
	}

  return state;
}

GAME_STATE gameLoop(SDL_Surface *screen)
{
	int i;
	GAME_STATE state = GAME_STATE_PLAY;
	SDL_Event event;

	Player pacman;
	World board;
	vector<Ghost> ghosts;
	ghosts.push_back(Ghost(20*20, 19*20));
	ghosts.push_back(Ghost(5*20, 19*20));

	pacman.initialize();
	board.initialize();
	for (i=0; i<ghosts.size(); i++)
	    ghosts[i].initialize();

	while (state == GAME_STATE_PLAY) 
	{
		//------------------------------------------------------------
		// Handle Events
		//------------------------------------------------------------
		while (SDL_PollEvent(&event) && state == GAME_STATE_PLAY) 
			state = ProcessEvents(state, event, pacman);

		//------------------------------------------------------------
		// Update Everything
		//------------------------------------------------------------
		for (i=0; i<ghosts.size(); i++)
			ghosts[i].update(board, pacman);
		pacman.update(board);
		state = board.update(state);

		//------------------------------------------------------------
		//Render Everything
		//------------------------------------------------------------
		//The background
		//    display_bmp("images/pac_background.bmp",screen);

		board.renderSDL(screen);
		pacman.renderSDL(screen);
		for (i=0; i<ghosts.size(); i++)
			ghosts[i].renderSDL(screen);

		SDL_Flip(screen);

		//------------------------------------------------------------
		//Check Here
		//------------------------------------------------------------
		for (i=0; i<ghosts.size(); i++)
			if (pow(ghosts[i].getRow()-pacman.getRow(),2)+pow(ghosts[i].getCol()-pacman.getCol(),2)<
				SPRITE_SIZE*SPRITE_SIZE)
					state = GAME_STATE_LOSE;
	}
 
	return state;
}
