#ifndef _STRUCTURES_H_
#define _STRUCTURES_H_

enum GAME_STATE 
{ 
	GAME_STATE_TITLE, 
	GAME_STATE_PLAY, 
	GAME_STATE_LOSE,
	GAME_STATE_WIN,
	GAME_STATE_QUIT 
};

enum PLAYER_ANIMATION_TYPE 
{ 
	PLAYER_RUN,
	PLAYER_IDLE,
	PLAYER_JUMP_START,
	PLAYER_JUMP_UP,
	PLAYER_JUMP_TOP,
	PLAYER_JUMP_DOWN,
	PLAYER_JUMP_LAND,
	PLAYER_TALK,
	PLAYER_SHOOT,
	PLAYER_SWORD,
	PLAYER_LADDER,
	PLAYER_JUMPSHOOT
};

enum GAME_IMAGES 
{ 
	CITY,		// 0
	UNDERGROUND,// 1
	BAR,		// 2
	RUN_START,	// 3
	RUN0,		// 4
	RUN1,		// 5
	RUN2,		// 6
	RUN3,		// 7
	RUN4,		// 8
	RUN5,		// 9
	RUN6,		// 10
	RUN7,		// 11
	RUN8,		// 12
	RUN9,		// 13
	IDLE0,		// 14
	IDLE1,		// 15
	IDLE2,		// 16
	JMP_START,
	JMP_UP_0,
	JMP_UP_1,
	JMP_TOP,
	JMP_DWN_0,
	JMP_DWN_1,
	JMP_LND_0,
	JMP_LND_1,
	SHOOT_0,
	SHOOT_1,
	SWORD_0,
	SWORD_1,
	SWORD_2,
	SWORD_3,
	SWORD_4,
	SWORD_5,
	SWORD_6,
	SWORD_7,
	SWORD_8,
	SWORD_9,
	SWORD_10,
	LADDER_0,
	LADDER_1,
	JUMPSHOT_0,
	JUMPSHOT_1,
	JUMPSHOT_2,
	JUMPSHOT_3,

	//WEAPON SHOT IMAGES
	FIREBALL_0,
	FIREBALL_1,
	FIREBALL_2,
	FIREBALL_3,

	//ENEMY ATTACKER0(Link) IMAGES
	ATTACKER0_IDLE0,
	ATTACKER0_WALK0,
	ATTACKER0_WALK1,
	ATTACKER0_WALK2,
	ATTACKER0_WALK3,
	ATTACKER0_SWORD0,
	ATTACKER0_SWORD1,
	ATTACKER0_DIE0,
	ATTACKER0_DIE1,
	ATTACKER0_DIE2,
	//ENEMY CRAWLER0(Mini-boss SMB3) IMAGES
	CRAWLER0_IDLE0,
	CRAWLER0_RUN_START,
	CRAWLER0_RUN0,
	CRAWLER0_RUN1,
	CRAWLER0_RUN2,
	//CRAWLER0_RUN3,
	//CRAWLER0_RUN4,
	CRAWLER0_ATTACK0,
	CRAWLER0_ATTACK1,
	CRAWLER0_ATTACK2,
	CRAWLER0_ATTACK3,
	CRAWLER0_ATTACK4,
	CRAWLER0_DIE0,
	CRAWLER0_DIE1,
	CRAWLER0_DIE2,
	//ENEMY SHOOTER0(Mario in fire suit) IMAGES
	SHOOTER0_IDLE0,
	SHOOTER0_WALK0,
	SHOOTER0_WALK1,
	SHOOTER0_SHOOT0,
	SHOOTER0_SHOOT1,
	SHOOTER0_DIE0
};

struct imgContainer
{
	SDL_Surface **sprite;
	int numFrames;
	bool isCopy;

	imgContainer(int numCells=0)
	{
		if(this->numFrames=numCells) this->sprite=NULL;
		else this->sprite=new SDL_Surface*[numCells];
		this->isCopy=false;
	}
	void define(int numCells=0)
	{
		if(this->numFrames!=0)
			delete [] this->sprite;

		if(this->numFrames=numCells) this->sprite=NULL;
		else this->sprite=new SDL_Surface*[numCells];
		this->isCopy=true;
	}
	//SDL_Surface **_sprite, int _numFrames
};

struct VECTOR
{
	double x;
	double y;
	double speed;
	double max;

	VECTOR()
	{
		this->x=0;
		this->y=0;
		this->speed=1;
		this->max=2;
	}

	VECTOR(int _speed, int _max)
	{
		this->x=0;
		this->y=0;
		if(_speed<1) _speed=1;
		if(_max<_speed)_max=_speed;

		this->speed=_speed;
		this->max=_max;
	}
	VECTOR(int _x, int _y, int _speed, int _max)
	{
		this->x=_x;
		this->y=_y;

		if(_speed<1) _speed=1;
		if(_max<_speed)_max=_speed;

		this->speed=_speed;
		this->max=_max;
	}

	VECTOR operator * (double _x)
	{
		return VECTOR(this->x*_x, this->y*_x);
	}

	VECTOR operator + (double _x)
	{
		return VECTOR(this->x+_x, this->y+_x);
	}

	VECTOR operator / (double _x)
	{
		if(_x==0) return VECTOR(0,0);

		VECTOR hld=VECTOR(this->x/_x, this->y/_x);
		if(this->x<0 && hld.x>-1) hld.x=-1;
		else if(this->x>0 && hld.x<1) hld.x=1;
		if(this->y<0 && hld.y>-1) hld.y=-1;
		else if(this->y>0 && hld.y<1) hld.y=1;

		return hld;
	}
};

struct coord
{
	double x;
	double y;

	coord()
	{
		this->x=this->y=0;
	}

	coord(double _x, double _y)
	{
		this->x=_x;
		this->y=_y;
	}

	double distance(coord xy)
	{
		return sqrt(pow(xy.x*1.0-this->x*1.0,2)+pow(xy.y*1.0-this->y*1.0,2));
	}

	double displacement()
	{
		return sqrt(pow(this->x*1.0,2)+pow(this->y*1.0,2));
	}

	coord operator +(coord xy)
	{
		return coord(this->x+xy.x, this->y+xy.y);
	}

	coord operator -(coord xy)
	{
		return coord(this->x-xy.x, this->y-xy.y);
	}

	coord operator *(coord xy)
	{
		return coord(this->x/xy.x, this->y/xy.y);
	}

	coord operator /(coord xy)
	{
		return coord(this->x*xy.x, this->y*xy.y);
	}

	coord operator + (VECTOR xy)
	{
		return coord(this->x+xy.x, this->y+xy.y);
	}

	coord operator - (VECTOR xy)
	{
		return coord(this->x-xy.x, this->y-xy.y);
	}
};

struct dimension
{
	int w;
	int h;

	dimension()
	{
		w=h=0;
	}

	dimension(int _w, int _h)
	{
		w=_w;
		h=_h;
	}
};


struct color
{
	int r;
	int g;
	int b;
	int a;

	color()
	{
		this->r=this->g=this->b=0;
		this->a=1;
	}

	color(int _r, int _g, int _b)
	{
		this->r=_r;
		this->g=_g;
		this->b=_b;
		this->a=1;
	}

	color(int _r, int _g, int _b, int _a)
	{
		this->r=_r;
		this->g=_g;
		this->b=_b;
		this->a=_a;
	}
};
#endif

// Quick fix to ashley's mistake
#ifndef _IMAGE_FUNCTIONS_
#define _IMAGE_FUNCTIONS_


Uint32 get_pixel32( SDL_Surface *surface, int x, int y )
{
    //Convert the pixels to 32 bit
    Uint32 *pixels = (Uint32 *)surface->pixels;

    //Get the requested pixel
    return pixels[ ( y * surface->w ) + x ];
}

void put_pixel32( SDL_Surface *surface, int x, int y, Uint32 pixel )
{
    //Convert the pixels to 32 bit
    Uint32 *pixels = (Uint32 *)surface->pixels;

    //Set the pixel
    pixels[ ( y * surface->w ) + x ] = pixel;
}

SDL_Surface *flip_surface( SDL_Surface *surface, int flags )
{
	//0x01 = flip vertical
	//0x02 = flip horizontal
    //Pointer to the soon to be flipped surface
    SDL_Surface *flipped = NULL;

    //If the image is color keyed
    if( surface->flags & SDL_SRCCOLORKEY )
    {
        flipped = SDL_CreateRGBSurface( SDL_SWSURFACE, surface->w, surface->h, surface->format->BitsPerPixel, surface->format->Rmask, surface->format->Gmask, surface->format->Bmask, 0 );
    }
    //Otherwise
    else
    {
        flipped = SDL_CreateRGBSurface( SDL_SWSURFACE, surface->w, surface->h, surface->format->BitsPerPixel, surface->format->Rmask, surface->format->Gmask, surface->format->Bmask, surface->format->Amask );
    }

    //If the surface must be locked
    if( SDL_MUSTLOCK( surface ) )
    {
        //Lock the surface
        SDL_LockSurface( surface );
    }

    //Go through columns
    for( int x = 0, rx = flipped->w - 1; x < flipped->w; x++, rx-- )
    {
        //Go through rows
        for( int y = 0, ry = flipped->h - 1; y < flipped->h; y++, ry-- )
        {
            //Get pixel
            Uint32 pixel = get_pixel32( surface, x, y );

            //Copy pixel
            if( ( flags & 0x01 ) && ( flags & 0x02 ) )
            {
                put_pixel32( flipped, rx, ry, pixel );
            }
            else if( flags & 0x02 )
            {
                put_pixel32( flipped, rx, y, pixel );
            }
            else if( flags & 0x01 )
            {
                put_pixel32( flipped, x, ry, pixel );
            }
        }
    }

    //Unlock surface
    if( SDL_MUSTLOCK( surface ) )
    {
        SDL_UnlockSurface( surface );
    }

    //Copy color key
    if( surface->flags & SDL_SRCCOLORKEY )
    {
        SDL_SetColorKey( flipped, SDL_RLEACCEL | SDL_SRCCOLORKEY, surface->format->colorkey );
    }

    //Return flipped surface
    return flipped;
}
#endif