#include "world.hpp"

World::World()
{
  rows = 24;
  cols = 32;

  grid = new int*[rows];
  grid[0] = new int [rows*cols];
  for (int r=1; r<rows; r++)
    grid[r] = &grid[0][r*cols];
}

World::~World()
{
  delete []grid[0];
  delete []grid;

  SDL_FreeSurface(wall_sprite);
  SDL_FreeSurface(empty_sprite);
  SDL_FreeSurface(pellet_sprite);
}

void World::initialize()
{
  boardPellets=0;
  int r, c;
  for (r=0; r<rows; r++)
    for (c=0; c<cols; c++)
      if (r==0 || c == 0 || r == rows-1 || c == cols-1)
	grid[r][c] = BOARD_WALL;
      else {
	grid[r][c] = BOARD_PELLET;
	boardPellets++;
      }

  //now for a stripe down the middle
  for (r=5; r<rows-5; r++)
    grid[r][cols/2] = BOARD_WALL;


  //now for sprites

  SDL_Surface *temp;

  temp = SDL_LoadBMP("images/wall.bmp");
  wall_sprite = SDL_DisplayFormat(temp);
  SDL_FreeSurface(temp);

  temp = SDL_LoadBMP("images/empty.bmp");
  empty_sprite = SDL_DisplayFormat(temp);
  SDL_FreeSurface(temp);

  temp = SDL_LoadBMP("images/pellet.bmp");
  pellet_sprite = SDL_DisplayFormat(temp);
  SDL_FreeSurface(temp);

}

GAME_STATE World::update(GAME_STATE state)
{
  if (state != GAME_STATE_PLAY)
    return state;
  if (boardPellets <=0)
    return GAME_STATE_WIN;
  return state;
}

void World::renderSDL(SDL_Surface *screen)
{
  int r, c;
  SDL_Rect block;

  //Draw World
  for (r=0; r<rows; r++)
    for (c=0; c<cols; c++) {
      block.x = c*SPRITE_SIZE;
      block.y = r*SPRITE_SIZE;
      if (grid[r][c]==BOARD_WALL)
	SDL_BlitSurface(wall_sprite, NULL, screen, &block);
      else if (grid[r][c]==BOARD_PELLET)
	SDL_BlitSurface(pellet_sprite, NULL, screen, &block);
      else 
 	SDL_BlitSurface(empty_sprite, NULL, screen, &block);
    }
}

bool World::isFree(double r, double c)
{
  int ir0 = int((r)/SPRITE_SIZE);
  int ir1 = int((r+SPRITE_SIZE-1)/SPRITE_SIZE);
  int ic0 = int((c)/SPRITE_SIZE);
  int ic1 = int((c+SPRITE_SIZE-1)/SPRITE_SIZE);

  if (ir0<0 || ir1>=rows || ic0<0 || ic1>=cols)
    return false;

  return (grid[ir0][ic0]!=BOARD_WALL &&
	  grid[ir0][ic1]!=BOARD_WALL &&
	  grid[ir1][ic0]!=BOARD_WALL &&
	  grid[ir1][ic1]!=BOARD_WALL);
}

bool World::eatPellet(double r, double c)
{
  int ir = int((r+SPRITE_SIZE/2)/SPRITE_SIZE);
  int ic = int((c+SPRITE_SIZE/2)/SPRITE_SIZE);

  if (ir<0 || ir>=rows || ic<0 || ic>=cols)
    return false;

  if (grid[ir][ic] == BOARD_PELLET) {
    grid[ir][ic] = BOARD_EMPTY;
    boardPellets--;
    return true;
  }
  return false;
}
