/*
 * I've tried to be pre-emptive on what containers you'll need and how you'll want to use them
 * Feel free to add new sturcters here, but be wary of modifying someone elses structure
*/

#ifndef _STRUCTURES_H_
#define _STRUCTURES_H_

// GAME_STATE defines what CAN be enumerated if you cast a variable to it
// Each name in the block will get a value from 0-N where N is the number of names in the listing
enum GAME_STATE 
{ 
	GAME_STATE_TITLE, 
	GAME_STATE_PLAY, 
	GAME_STATE_LOSE,
	GAME_STATE_WIN,
	GAME_STATE_QUIT 
};

/*
Each class will actually needs its own state listing since some states don't exist
enum ANIMATION_STATE
{
	JUMP,
	WALK,
	RUN,
	STAND
};
*/

// This is the main container of all ANIMATIONS
struct imgContainer
{
	SDL_Surface **sprite;	// The animation has an array of images/frames
	int numFrames;			// max number of frames in this animation
	bool isCopy;			// is the animation array a copy

// CONSTRUCTOR
	imgContainer(int numCells=0)
	{
		if(this->numFrames=numCells) this->sprite=NULL;	// If (void) contstructed, null sprite pointer
		else this->sprite=new SDL_Surface*[numCells];   // Otherwise reserve the number of frames
		this->isCopy=false;								// Orginal Image
	}

/// DESTRUCTOR
	~imgContainer()
	{
		// delete each sprite
		for(int i=0; i<this->numFrames; i++)
			delete this->sprite[i];

		// delete the sprite container
		delete [] this->sprite;
	}

// MUTATOR
	void define(int numCells=0)   // this just allows you to regenearate the object
	{
		if(this->numFrames!=0)
		{
			// delete each sprite
			for(int i=0; i<this->numFrames; i++)
				delete this->sprite[i];

			// delete the sprite container
			delete [] this->sprite;
		}

		if(this->numFrames=numCells) this->sprite=NULL;	// If (void) contstructed, null sprite pointer
		else this->sprite=new SDL_Surface*[numCells];   // Otherwise reserve the number of frames
		this->isCopy=false;								// Orginal Image
	}
};

// The VECTOR is used to modify COORDs
struct VECTOR
{
	int x;
	int y;
	int speed;
	int max;

// CONSTRUCTOR
	VECTOR()
	{
		this->x=0;
		this->y=0;
		this->speed=1;
		this->max=2;
	}

	VECTOR(int _speed, int _max)
	{
		this->x=0;
		this->y=0;
		if(_speed<1) _speed=1;
		if(_max<_speed)_max=_speed;

		this->speed=_speed;
		this->max=_max;
	}

// DESCTRUCTOR {no need for one}

// MUTATORS
	VECTOR operator * (int x)							// Allows changing the speed by a scalor; good for '2x' effects
	{
		return VECTOR(this->speed*x, this->max*x);
	}

	VECTOR operator / (int x)                           // Allows changing the speed by a scalor; good for '2x' effects
	{
		return VECTOR(this->speed/x, this->max/x);
	}

	double magSq()										// This will give you the magnitude squared; because sqrt() is intensive
	{
		return pow((double)this->x,2)+pow((double)this->y,2);
	}

	double mag()                                        // Incase you want the computer to do the extra work; a normal magnitude
	{
		return sqrt(pow((double)this->x,2)+pow((double)this->y,2));
	}
};

// This will allow simple XY storage and modifiers
struct coord
{
	int x;
	int y;

// CONSTRUCTOR
	coord()
	{
		this->x=this->y=0;
	}

	coord(int _x, int _y)
	{
		this->x=_x;
		this->y=_y;
	}

// DESTRUCTOR {not needed}

// ACCESSORS
	double distanceSq(coord xy)
	{
		return pow(xy.x*1.0-this->x*1.0,2)+pow(xy.y*1.0-this->y*1.0,2);
	}

	double distance(coord xy)
	{
		return sqrt(pow(xy.x*1.0-this->x*1.0,2)+pow(xy.y*1.0-this->y*1.0,2));
	}

	double displacement()
	{
		return sqrt(pow(this->x*1.0,2)+pow(this->y*1.0,2));
	}

// MUTATORS
	coord operator +(coord xy)
	{
		return coord(this->x+xy.x, this->y+xy.y);
	}

	coord operator -(coord xy)
	{
		return coord(this->x-xy.x, this->y-xy.y);
	}

	coord operator *(coord xy)
	{
		return coord(this->x/xy.x, this->y/xy.y);
	}

	coord operator /(coord xy)
	{
		return coord(this->x*xy.x, this->y*xy.y);
	}

	// THIS ALLOWS YOU TO ADD YOUR VECTORS TO THE POSITION
	coord operator + (VECTOR xy)
	{
		return coord(this->x+xy.x, this->y+xy.y);
	}
};

// DEPRICATED
struct dimension
{
	int w;
	int h;

	dimension()
	{
		w=h=0;
	}

	dimension(int _w, int _h)
	{
		w=_w;
		h=_h;
	}
};

// DEPRICATED
struct color
{
	int r;
	int g;
	int b;
	int a;

	color()
	{
		this->r=this->g=this->b=0;
		this->a=1;
	}

	color(int _r, int _g, int _b)
	{
		this->r=_r;
		this->g=_g;
		this->b=_b;
		this->a=1;
	}

	color(int _r, int _g, int _b, int _a)
	{
		this->r=_r;
		this->g=_g;
		this->b=_b;
		this->a=_a;
	}
};

#endif